@@ -2,7 +2,7 @@
 /*
 
 cellwriter -- a character recognition input method
-Copyright (C) 2007 Michael Levin <risujin@risujin.org>
+Copyright (C) 2007 Michael Levin <risujin@gmail.com>
 
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
@@ -27,9 +27,6 @@ Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 #include <signal.h>
 #include <stdio.h>
 #include <errno.h>
-#ifdef HAVE_GNOME
-#include <libgnome/libgnome.h>
-#endif
 
 /* recognize.c */
 extern int strength_sum;
@@ -375,9 +372,9 @@ static int move_file(char *from, char *to)
                 gsize bytes_read, bytes_written;
 
                 /* Read a block in */
-                g_io_channel_read_chars(src_channel, buffer, sizeof (buffer),
-                                        &bytes_read, &error);
-                if (bytes_read < 1 || error)
+                if (g_io_channel_read_chars(src_channel, buffer,
+                                            sizeof (buffer), &bytes_read,
+                                            &error) != G_IO_STATUS_NORMAL)
                         break;
 
                 /* Write the block out */
@@ -618,6 +615,7 @@ int profile_line, log_level = 4;
 
 static char *log_filename = NULL;
 static FILE *log_file = NULL;
+static int ignore_fifo;
 
 /* Profile commands table */
 static struct {
@@ -662,6 +660,8 @@ static GOptionEntry command_line_opts[] = {
           "Do not save changes to the profile", NULL },
         { "disable-overwrite", 0, 0, G_OPTION_ARG_NONE, &key_disable_overwrite,
           "Do not modify the keymap", NULL },
+        { "ignore-fifo", 0, 0, G_OPTION_ARG_NONE, &ignore_fifo,
+          "Allow starting a second instance", NULL },
 
         /* Sentinel */
         { NULL, 0, 0, 0, NULL, NULL, NULL }
@@ -836,7 +836,7 @@ static void second_instance(char *str)
                 window_hide();
         else if (str[0] == '1' || str[0] == 'S' || str[0] == 's')
                 window_show();
-        else if (str[0] == 'T' || str[0] == 't')
+        else if (str[0] == '2' || str[0] == 'T' || str[0] == 't')
                 window_toggle();
 }
 
@@ -848,7 +848,8 @@ int main(int argc, char *argv[])
         /* Initialize GTK+ */
         error = NULL;
         if (!gtk_init_with_args(&argc, &argv,
-                                "grid-entry handwriting input panel",
+                                "- Grid-entry handwriting input panel "
+                                "(version " PACKAGE_VERSION ")",
                                 command_line_opts, NULL, &error))
                 return 0;
 
@@ -866,19 +867,21 @@ int main(int argc, char *argv[])
         /* See if the program is already running */
         g_message("Starting " PACKAGE_STRING);
         create_user_dir();
-        if (!window_embedded &&
-            single_instance_init((SingleInstanceFunc)second_instance,
-                                 window_force_hide ? "0" : "1")) {
-                gdk_notify_startup_complete();
-                g_message(PACKAGE_NAME " already running, quitting");
-                return 0;
-        }
+        if (!window_embedded && !ignore_fifo) {
+                const char *msg;
 
-#ifdef HAVE_GNOME
-        /* Initialize GNOME for the Help button */
-        gnome_program_init(PACKAGE, VERSION, LIBGNOME_MODULE,
-                           argc, argv, NULL);
-#endif
+                msg = "2";
+                if (window_force_show)
+                        msg = "1";
+                else if (window_force_hide)
+                        msg = "0";
+                if (single_instance_init((SingleInstanceFunc)second_instance,
+                                         msg)) {
+                        gdk_notify_startup_complete();
+                        g_message(PACKAGE_NAME " already running, quitting");
+                        return 0;
+                }
+        }
 
         /* Component initilization */
         if (key_event_init()) {
@@ -968,4 +971,3 @@ int main(int argc, char *argv[])
 
         return 0;
 }
-
