@@ -2,7 +2,7 @@
 /*
 
 cellwriter -- a character recognition input method
-Copyright (C) 2007 Michael Levin <risujin@risujin.org>
+Copyright (C) 2007 Michael Levin <risujin@gmail.com>
 
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
@@ -28,6 +28,11 @@ Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 #include <X11/Xlib.h>
 #include <gdk/gdkx.h>
 
+/* Enable this option if your window manager support window gravity. With this
+   option, the CellWriter window will resize from the bottom using the
+   window manager rather than manually. */
+/* #define WINDOW_GRAVITY */
+
 /* options.c */
 void options_dialog_open(void);
 
@@ -74,8 +79,8 @@ int window_force_x = -1, window_force_y = -1, training_block = 0,
 static char *tab_xpm[] =
 {
         "7 4 2 1",
-        " 	c None",
-        ".	c #000000",
+        "         c None",
+        ".        c #000000",
         ".......",
         " ..... ",
         "  ...  ",
@@ -140,15 +145,15 @@ static void update_struts(void)
          scan down from the top of the screen. GOK and other applications
          somehow get around this but I can't figure out how. */
 {
-	static guint32 struts[12];
-	guint32 new2 = 0, new3 = 0, new9 = 0, new11 = 0;
-	GdkAtom atom_strut, atom_strut_partial, cardinal;
+        static gulong struts[12];
+        guint32 new2 = 0, new3 = 0, new9 = 0, new11 = 0;
+        GdkAtom atom_strut, atom_strut_partial, cardinal;
 
-	if (!window || !window->window || !window_struts)
-	        return;
-	if (window_docked == WINDOW_DOCKED_TOP) {
-	        new2 = window_frame.y + window_frame.height;
-	        new9 = window_frame.width;
+        if (!window || !window->window || !window_struts)
+                return;
+        if (window_docked == WINDOW_DOCKED_TOP) {
+                new2 = window_frame.y + window_frame.height;
+                new9 = window_frame.width;
         } else if (window_docked == WINDOW_DOCKED_BOTTOM) {
                 new3 = window_frame.height;
                 new11 = window_frame.width;
@@ -161,14 +166,14 @@ static void update_struts(void)
         struts[3] = new3;
         struts[9] = new9;
         struts[11] = new11;
-	atom_strut = gdk_atom_intern("_NET_WM_STRUT", FALSE),
-	atom_strut_partial = gdk_atom_intern("_NET_WM_STRUT_PARTIAL", FALSE);
-	cardinal = gdk_atom_intern("CARDINAL", FALSE);
-	gdk_property_change(GDK_WINDOW(window->window), atom_strut, cardinal,
-        	            32, GDK_PROP_MODE_REPLACE, (guchar*)&struts, 4);
-	gdk_property_change(GDK_WINDOW(window->window), atom_strut_partial,
-	                    cardinal, 32, GDK_PROP_MODE_REPLACE,
-	                    (guchar*)&struts, 12);
+        atom_strut = gdk_atom_intern("_NET_WM_STRUT", FALSE),
+        atom_strut_partial = gdk_atom_intern("_NET_WM_STRUT_PARTIAL", FALSE);
+        cardinal = gdk_atom_intern("CARDINAL", FALSE);
+        gdk_property_change(GDK_WINDOW(window->window), atom_strut, cardinal,
+                            32, GDK_PROP_MODE_REPLACE, (guchar*)&struts, 4);
+        gdk_property_change(GDK_WINDOW(window->window), atom_strut_partial,
+                            cardinal, 32, GDK_PROP_MODE_REPLACE,
+                            (guchar*)&struts, 12);
 }
 
 static void set_geometry_hints(void)
@@ -189,17 +194,15 @@ static void set_geometry_hints(void)
                                       GDK_HINT_MIN_SIZE | GDK_HINT_MAX_SIZE);
         trace("%dx%d", geometry.min_width, geometry.min_height);
 
+#ifdef WINDOW_GRAVITY
         /* In some bright and sunny alternate universe when specifications are
            actually implemented as inteded, this function alone would cause the
            window frame to expand upwards without having to perform the ugly
-           hack in window_configure(). XFWM4 does not respect this hint and
-           setting this hint will further mess up the window_configure()
-           movement code. */
-        /*geometry.win_gravity = window_docked == WINDOW_DOCKED_TOP ?
-                                 GDK_GRAVITY_NORTH_WEST :
-                                 GDK_GRAVITY_SOUTH_WEST;
+           hack in window_configure(). */
+        geometry.win_gravity = GDK_GRAVITY_SOUTH_WEST;
         gtk_window_set_geometry_hints(GTK_WINDOW(window), window, &geometry,
-                                      GDK_HINT_WIN_GRAVITY);*/
+                                      GDK_HINT_WIN_GRAVITY);
+#endif
 }
 
 static void docked_move_resize(void)
@@ -217,6 +220,9 @@ static void docked_move_resize(void)
         cell_widget_pack();
         key_widget_resize(key_widget);
         trace("y=%d", y);
+        if (window_struts)
+                gdk_window_set_type_hint(GDK_WINDOW(window->window),
+                                         GDK_WINDOW_TYPE_HINT_DOCK);
 }
 
 static gboolean window_configure(GtkWidget *widget, GdkEventConfigure *event)
@@ -225,12 +231,11 @@ static gboolean window_configure(GtkWidget *widget, GdkEventConfigure *event)
 {
         GdkRectangle new_frame = {0, 0, 0, 0};
         GdkScreen *screen;
-        GdkDisplay *display;
         int screen_w, screen_h, height_change, label_w;
 
         if (!window || !window->window)
                 return FALSE;
-        display = gtk_widget_get_display(window);
+        gtk_widget_get_display(window);
 
         /* Get screen and window information */
         screen = gtk_window_get_screen(GTK_WINDOW(window));
@@ -266,6 +271,7 @@ static gboolean window_configure(GtkWidget *widget, GdkEventConfigure *event)
                 return FALSE;
         }
 
+#ifndef WINDOW_GRAVITY
         /* Keep the window aligned to the bottom border */
         if (height_change && window_frame.y + window_frame.height / 2 >
                              gdk_screen_get_height(screen) / 2)
@@ -299,6 +305,7 @@ static gboolean window_configure(GtkWidget *widget, GdkEventConfigure *event)
                 trace("moving to (%d, %d)", window_frame.x, window_frame.y);
         } else
                 window_frame = new_frame;
+#endif
 
         return FALSE;
 }
@@ -317,6 +324,21 @@ void window_set_docked(int mode)
         cell_widget_pack();
         key_widget_resize(key_widget);
 
+        /* Set window docking hints.
+           FIXME This allegedly solves docking problems with some window
+                 managers but only seems to cause more problems for
+                 Compiz and Metacity. */
+        /*
+        gtk_widget_hide(window);
+        if (mode == WINDOW_UNDOCKED)
+                gtk_window_set_type_hint(GTK_WINDOW(window),
+                                         GDK_WINDOW_TYPE_HINT_NORMAL);
+        else
+                gtk_window_set_type_hint(GTK_WINDOW(window),
+                                         GDK_WINDOW_TYPE_HINT_DOCK);
+        gtk_widget_show(window);
+        */
+
         /* Restore the old window position */
         if (!mode) {
                 update_struts();
@@ -481,6 +503,7 @@ gboolean window_close(void)
                 key_widget_cleanup(key_widget);
                 return TRUE;
         }
+        g_debug("Status icon failed to embed, quitting.");
         return FALSE;
 }
 
@@ -542,6 +565,15 @@ static void print_window_xid(GtkWidget *widget)
         g_print("%d\n", (unsigned int)GDK_WINDOW_XID(widget->window));
 }
 
+static gint status_icon_embedded_check() {
+        if (!status_icon_embedded()) {
+                g_debug("Status icon failed to embed, showing window.");
+                window_shown = TRUE;
+                gtk_widget_show(window);
+        }
+        return FALSE;
+}
+
 void window_create(void)
 /* Create the main window and child widgets */
 {
@@ -562,11 +594,6 @@ void window_create(void)
         gtk_window_set_accept_focus(GTK_WINDOW(window), FALSE);
         gtk_window_set_resizable(GTK_WINDOW(window), FALSE);
 
-        /* This hint was alleged to fix the strut problems with metacity but
-           doesn't and only causes the window to overlap the docked panels */
-        /*gtk_window_set_type_hint(GTK_WINDOW(window),
-                                 GDK_WINDOW_TYPE_HINT_DOCK);*/
-
         /* Tooltips */
         tooltips = gtk_tooltips_new();
         gtk_tooltips_enable(tooltips);
@@ -777,8 +804,13 @@ void window_create(void)
                 window_shown = FALSE;
         else if (window_force_show)
                 window_shown = TRUE;
-        if (window_shown || !status_icon_embedded())
+        if (window_shown) {
                 gtk_widget_show(window);
+        } else {
+                /* Check if the status icon is embedded after a timeout so that
+                   it has a chance to embed itself into the tray. */
+                gtk_timeout_add(10, status_icon_embedded_check, NULL);
+        }
 }
 
 void window_sync(void)
@@ -941,8 +973,30 @@ UnicodeBlock unicode_blocks[] =
         { FALSE, 0xFE70, 0xFEFF, "Arabic Presentation Forms-B" },
         { FALSE, 0xFF00, 0xFFEF, "Halfwidth and Fullwidth Forms" },
         { FALSE, 0xFFF0, 0xFFFF, "Specials" },
+        { FALSE, 0x10000, 0x1007F, "Linear B Syllabary" },
+        { FALSE, 0x10080, 0x100FF, "Linear B Ideograms" },
+        { FALSE, 0x10100, 0x1013F, "Aegean Numbers" },
+        { FALSE, 0x10140, 0x1018F, "Ancient Greek Numbers" },
+        { FALSE, 0x10300, 0x1032F, "Old Italic" },
+        { FALSE, 0x10330, 0x1034F, "Gothic" },
+        { FALSE, 0x10380, 0x1039F, "Ugaritic" },
+        { FALSE, 0x103A0, 0x103DF, "Old Persian" },
+        { FALSE, 0x10400, 0x1044F, "Deseret" },
+        { FALSE, 0x10450, 0x1047F, "Shavian" },
+        { FALSE, 0x10480, 0x104AF, "Osmanya" },
+        { FALSE, 0x10800, 0x1083F, "Cypriot Syllabary" },
+        { FALSE, 0x10900, 0x1091F, "Phoenician" },
+        { FALSE, 0x10A00, 0x10A5F, "Kharoshthi" },
+        { FALSE, 0x12000, 0x123FF, "Cuneiform" },
+        { FALSE, 0x12400, 0x1247F, "Cuneiform Numbers and Punctuation" },
+        { FALSE, 0x1D000, 0x1D0FF, "Byzantine Musical Symbols" },
+        { FALSE, 0x1D100, 0x1D1FF, "Musical Symbols" },
+        { FALSE, 0x1D200, 0x1D24F, "Ancient Greek Musical Notation" },
+        { FALSE, 0x1D300, 0x1D35F, "Tai Xuan Jing Symbols" },
+        { FALSE, 0x1D360, 0x1D37F, "Counting Rod Numerals" },
+        { FALSE, 0x1D400, 0x1D7FF, "Mathematical Alphanumeric Symbols" },
 
-        /* Cut the table here because we only support 4-byte characters */
+        /* Cut the table here because the rest are non-printable characters */
         { FALSE, 0,      0,      NULL },
 };
 
@@ -991,10 +1045,10 @@ void unicode_block_toggle(int block, int on)
 */
 
 #define WELCOME_MSG "You are either starting " PACKAGE_NAME " for the first " \
-                    "time or have not yet created any training samples.\n\n" \
+                    "time or have not yet created any training samples. " \
                     PACKAGE_NAME " requires accurate training samples of " \
-                    "your characters before it can work.\n\n" \
-                    PACKAGE_NAME " will now enter training mode. " \
+                    "your characters before it can work. " \
+                    "The program will now enter training mode. " \
                     "Carefully draw each character in its cell and then " \
                     "press the 'Train' button."
 
@@ -1017,4 +1071,3 @@ void startup_splash_show(void)
         /* Press in the training button for the user */
         gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(train_button), TRUE);
 }
-
